# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import mock
import testtools

import shade
from shade.tests.unit import base


class TestVolumeAccess(base.TestCase):
    @mock.patch.object(shade.OpenStackCloud, 'cinder_client')
    def test_list_volume_types(self, mock_cinder):
        volume_type = dict(
            id='voltype01', description='volume type description',
            name='name', is_public=False)
        mock_cinder.volume_types.list.return_value = [volume_type]

        self.assertTrue(self.cloud.list_volume_types())

    @mock.patch.object(shade.OpenStackCloud, 'cinder_client')
    def test_get_volume_type(self, mock_cinder):
        volume_type = dict(
            id='voltype01', description='volume type description', name='name',
            is_public=False)
        mock_cinder.volume_types.list.return_value = [volume_type]

        volume_type_got = self.cloud.get_volume_type('name')
        self.assertEqual(volume_type_got.id, volume_type['id'])

    @mock.patch.object(shade.OpenStackCloud, 'cinder_client')
    def test_get_volume_type_access(self, mock_cinder):
        volume_type = dict(
            id='voltype01', description='volume type description', name='name',
            is_public=False)
        volume_type_access = [
            dict(volume_type_id='voltype01', name='name', project_id='prj01'),
            dict(volume_type_id='voltype01', name='name', project_id='prj02')
        ]
        mock_cinder.volume_types.list.return_value = [volume_type]
        mock_cinder.volume_type_access.list.return_value = volume_type_access

        self.assertEqual(
            len(self.op_cloud.get_volume_type_access('name')), 2)

    @mock.patch.object(shade.OpenStackCloud, 'cinder_client')
    def test_remove_volume_type_access(self, mock_cinder):
        volume_type = dict(
            id='voltype01', description='volume type description', name='name',
            is_public=False)
        project_001 = dict(volume_type_id='voltype01', name='name',
                           project_id='prj01')
        project_002 = dict(volume_type_id='voltype01', name='name',
                           project_id='prj02')
        volume_type_access = [project_001, project_002]
        mock_cinder.volume_types.list.return_value = [volume_type]
        mock_cinder.volume_type_access.list.return_value = volume_type_access

        def _fake_remove(*args, **kwargs):
            volume_type_access.pop()

        mock_cinder.volume_type_access.remove_project_access.side_effect = \
            _fake_remove

        self.assertEqual(
            len(self.op_cloud.get_volume_type_access(
                volume_type['name'])), 2)
        self.op_cloud.remove_volume_type_access(
            volume_type['name'], project_001['project_id'])

        self.assertEqual(
            len(self.op_cloud.get_volume_type_access('name')), 1)

    @mock.patch.object(shade.OpenStackCloud, 'cinder_client')
    def test_add_volume_type_access(self, mock_cinder):
        volume_type = dict(
            id='voltype01', description='volume type description', name='name',
            is_public=False)
        project_001 = dict(volume_type_id='voltype01', name='name',
                           project_id='prj01')
        project_002 = dict(volume_type_id='voltype01', name='name',
                           project_id='prj02')
        volume_type_access = [project_001]
        mock_cinder.volume_types.list.return_value = [volume_type]
        mock_cinder.volume_type_access.list.return_value = volume_type_access
        mock_cinder.volume_type_access.add_project_access.return_value = None

        def _fake_add(*args, **kwargs):
            volume_type_access.append(project_002)

        mock_cinder.volume_type_access.add_project_access.side_effect = \
            _fake_add

        self.op_cloud.add_volume_type_access(
            volume_type['name'], project_002['project_id'])
        self.assertEqual(
            len(self.op_cloud.get_volume_type_access('name')), 2)

    @mock.patch.object(shade.OpenStackCloud, 'cinder_client')
    def test_add_volume_type_access_missing(self, mock_cinder):
        volume_type = dict(
            id='voltype01', description='volume type description', name='name',
            is_public=False)
        project_001 = dict(volume_type_id='voltype01', name='name',
                           project_id='prj01')
        mock_cinder.volume_types.list.return_value = [volume_type]
        with testtools.ExpectedException(shade.OpenStackCloudException,
                                         "VolumeType not found: MISSING"):
            self.op_cloud.add_volume_type_access(
                "MISSING", project_001['project_id'])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

import shade
from shade.tests.unit import base
from testtools import matchers


RAW_ROLE_ASSIGNMENTS = [
    {
        "links": {"assignment": "http://example"},
        "role": {"id": "123456"},
        "scope": {"domain": {"id": "161718"}},
        "user": {"id": "313233"}
    },
    {
        "links": {"assignment": "http://example"},
        "group": {"id": "101112"},
        "role": {"id": "123456"},
        "scope": {"project": {"id": "456789"}}
    }
]


class TestIdentityRoles(base.RequestsMockTestCase):

    def get_mock_url(self, service_type='identity', interface='admin',
                     resource='roles', append=None, base_url_append='v3',
                     qs_elements=None):
        return super(TestIdentityRoles, self).get_mock_url(
            service_type, interface, resource, append, base_url_append,
            qs_elements)

    def test_list_roles(self):
        self._add_discovery_uri_call()
        role_data = self._get_role_data()
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'roles': [role_data.json_response['role']]})
        ])
        self.op_cloud.list_roles()
        self.assert_calls()

    def test_get_role_by_name(self):
        self._add_discovery_uri_call()
        role_data = self._get_role_data()
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'roles': [role_data.json_response['role']]})
        ])
        role = self.op_cloud.get_role(role_data.role_name)

        self.assertIsNotNone(role)
        self.assertThat(role.id, matchers.Equals(role_data.role_id))
        self.assertThat(role.name, matchers.Equals(role_data.role_name))
        self.assert_calls()

    def test_get_role_by_id(self):
        self._add_discovery_uri_call()
        role_data = self._get_role_data()
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'roles': [role_data.json_response['role']]})
        ])
        role = self.op_cloud.get_role(role_data.role_id)

        self.assertIsNotNone(role)
        self.assertThat(role.id, matchers.Equals(role_data.role_id))
        self.assertThat(role.name, matchers.Equals(role_data.role_name))
        self.assert_calls()

    def test_create_role(self):
        self._add_discovery_uri_call()
        role_data = self._get_role_data()
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json=role_data.json_response,
                 validate=role_data.json_request),
            dict(method='GET',
                 uri=self.get_mock_url(append=[role_data.role_id]),
                 status_code=200,
                 json=role_data.json_response)
        ])

        role = self.op_cloud.create_role(role_data.role_name)

        self.assertIsNotNone(role)
        self.assertThat(role.name, matchers.Equals(role_data.role_name))
        self.assertThat(role.id, matchers.Equals(role_data.role_id))
        self.assert_calls()

    def test_delete_role_by_id(self):
        self._add_discovery_uri_call()
        role_data = self._get_role_data()
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'roles': [role_data.json_response['role']]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(append=[role_data.role_id]),
                 status_code=204)
        ])
        role = self.op_cloud.delete_role(role_data.role_id)
        self.assertThat(role, matchers.Equals(True))
        self.assert_calls()

    def test_delete_role_by_name(self):
        self._add_discovery_uri_call()
        role_data = self._get_role_data()
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'roles': [role_data.json_response['role']]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(append=[role_data.role_id]),
                 status_code=204)
        ])
        role = self.op_cloud.delete_role(role_data.role_name)
        self.assertThat(role, matchers.Equals(True))
        self.assert_calls()

    def test_list_role_assignments(self):
        self._add_discovery_uri_call()
        domain_data = self._get_domain_data()
        user_data = self._get_user_data(domain_id=domain_data.domain_id)
        group_data = self._get_group_data(domain_id=domain_data.domain_id)
        project_data = self._get_project_data(domain_id=domain_data.domain_id)
        role_data = self._get_role_data()
        response = [
            {'links': 'https://example.com',
             'role': {'id': role_data.role_id},
             'scope': {'domain': {'id': domain_data.domain_id}},
             'user': {'id': user_data.user_id}},
            {'links': 'https://example.com',
             'role': {'id': role_data.role_id},
             'scope': {'project': {'id': project_data.project_id}},
             'group': {'id': group_data.group_id}},
        ]
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     resource='role_assignments'),
                 status_code=200,
                 json={'role_assignments': response},
                 complete_qs=True)
        ])
        ret = self.op_cloud.list_role_assignments()
        self.assertThat(len(ret), matchers.Equals(2))
        self.assertThat(ret[0].user, matchers.Equals(user_data.user_id))
        self.assertThat(ret[0].id, matchers.Equals(role_data.role_id))
        self.assertThat(ret[0].domain, matchers.Equals(domain_data.domain_id))
        self.assertThat(ret[1].group, matchers.Equals(group_data.group_id))
        self.assertThat(ret[1].id, matchers.Equals(role_data.role_id))
        self.assertThat(ret[1].project,
                        matchers.Equals(project_data.project_id))

    def test_list_role_assignments_filters(self):
        self._add_discovery_uri_call()
        domain_data = self._get_domain_data()
        user_data = self._get_user_data(domain_id=domain_data.domain_id)
        role_data = self._get_role_data()
        response = [
            {'links': 'https://example.com',
             'role': {'id': role_data.role_id},
             'scope': {'domain': {'id': domain_data.domain_id}},
             'user': {'id': user_data.user_id}}
        ]

        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     resource='role_assignments',
                     qs_elements=['scope.domain.id=%s' % domain_data.domain_id,
                                  'user.id=%s' % user_data.user_id,
                                  'effective=True']),
                 status_code=200,
                 json={'role_assignments': response},
                 complete_qs=True)
        ])
        params = dict(user=user_data.user_id, domain=domain_data.domain_id,
                      effective=True)
        ret = self.op_cloud.list_role_assignments(filters=params)
        self.assertThat(len(ret), matchers.Equals(1))
        self.assertThat(ret[0].user, matchers.Equals(user_data.user_id))
        self.assertThat(ret[0].id, matchers.Equals(role_data.role_id))
        self.assertThat(ret[0].domain, matchers.Equals(domain_data.domain_id))

    def test_list_role_assignments_exception(self):
        self._add_discovery_uri_call()
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(resource='role_assignments'),
                 status_code=403)
        ])
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Failed to list role assignments"
        ):
            self.op_cloud.list_role_assignments()
        self.assert_calls()

    def test_list_role_assignments_keystone_v2(self):
        self.use_keystone_v2()
        role_data = self._get_role_data()
        user_data = self._get_user_data()
        project_data = self._get_project_data(v3=False)
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     resource='tenants',
                     append=[project_data.project_id,
                             'users',
                             user_data.user_id,
                             'roles'],
                     base_url_append=None),
                 status_code=200,
                 json={'roles': [role_data.json_response['role']]})
        ])
        ret = self.op_cloud.list_role_assignments(
            filters={
                'user': user_data.user_id,
                'project': project_data.project_id})
        self.assertThat(len(ret), matchers.Equals(1))
        self.assertThat(ret[0].project,
                        matchers.Equals(project_data.project_id))
        self.assertThat(ret[0].id, matchers.Equals(role_data.role_id))
        self.assertThat(ret[0].user, matchers.Equals(user_data.user_id))
        self.assert_calls()

    def test_list_role_assignments_keystone_v2_with_role(self):
        self.use_keystone_v2()
        roles_data = [self._get_role_data() for r in range(0, 2)]
        user_data = self._get_user_data()
        project_data = self._get_project_data(v3=False)
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     resource='tenants',
                     append=[project_data.project_id,
                             'users',
                             user_data.user_id,
                             'roles'],
                     base_url_append=None),
                 status_code=200,
                 json={'roles': [r.json_response['role'] for r in roles_data]})
        ])
        ret = self.op_cloud.list_role_assignments(
            filters={
                'role': roles_data[0].role_id,
                'user': user_data.user_id,
                'project': project_data.project_id})
        self.assertThat(len(ret), matchers.Equals(1))
        self.assertThat(ret[0].project,
                        matchers.Equals(project_data.project_id))
        self.assertThat(ret[0].id, matchers.Equals(roles_data[0].role_id))
        self.assertThat(ret[0].user, matchers.Equals(user_data.user_id))
        self.assert_calls()

    def test_list_role_assignments_exception_v2(self):
        self.use_keystone_v2()
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Must provide project and user for keystone v2"
        ):
            self.op_cloud.list_role_assignments()
            self.assert_calls()

    def test_list_role_assignments_exception_v2_no_project(self):
        self.use_keystone_v2()
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Must provide project and user for keystone v2"
        ):
            self.op_cloud.list_role_assignments(filters={'user': '12345'})
            self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

import shade
from shade.tests.unit import base


class TestVolumeBackups(base.TestCase):
    @mock.patch.object(shade.OpenStackCloud, 'list_volume_backups')
    @mock.patch("shade._utils._filter_list")
    def test_search_volume_backups(self, m_filter_list, m_list_volume_backups):
        result = self.cloud.search_volume_backups(
            mock.sentinel.name_or_id, mock.sentinel.filter)

        m_list_volume_backups.assert_called_once_with()
        m_filter_list.assert_called_once_with(
            m_list_volume_backups.return_value, mock.sentinel.name_or_id,
            mock.sentinel.filter)
        self.assertIs(m_filter_list.return_value, result)

    @mock.patch("shade._utils._get_entity")
    def test_get_volume_backup(self, m_get_entity):
        result = self.cloud.get_volume_backup(
            mock.sentinel.name_or_id, mock.sentinel.filter)

        self.assertIs(m_get_entity.return_value, result)
        m_get_entity.assert_called_once_with(
            self.cloud.search_volume_backups, mock.sentinel.name_or_id,
            mock.sentinel.filter)

    @mock.patch.object(shade.OpenStackCloud, 'cinder_client')
    def test_list_volume_backups(self, m_cinder_client):
        backup_id = '6ff16bdf-44d5-4bf9-b0f3-687549c76414'
        m_cinder_client.backups.list.return_value = [
            {'id': backup_id}
        ]
        result = self.cloud.list_volume_backups(
            mock.sentinel.detailed, mock.sentinel.search_opts)

        m_cinder_client.backups.list.assert_called_once_with(
            detailed=mock.sentinel.detailed,
            search_opts=mock.sentinel.search_opts)
        self.assertEqual(backup_id, result[0]['id'])

    @mock.patch.object(shade.OpenStackCloud, 'cinder_client')
    @mock.patch("shade._utils._iterate_timeout")
    @mock.patch.object(shade.OpenStackCloud, 'get_volume_backup')
    def test_delete_volume_backup(self, m_get_volume_backup,
                                  m_iterate_timeout, m_cinder_client):
        m_get_volume_backup.side_effect = [{'id': 42}, True, False]
        self.cloud.delete_volume_backup(
            mock.sentinel.name_or_id, mock.sentinel.force, mock.sentinel.wait,
            mock.sentinel.timeout)

        m_iterate_timeout.assert_called_once_with(
            mock.sentinel.timeout, mock.ANY)
        m_cinder_client.backups.delete.assert_called_once_with(
            backup=42, force=mock.sentinel.force)

        # We expect 3 calls, the last return_value is False which breaks the
        # wait loop.
        m_get_volume_backup.call_args_list = [mock.call(42)] * 3



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_usage
----------------------------------

Functional tests for `shade` usage method
"""
import datetime

from shade.tests.functional import base


class TestUsage(base.BaseFunctionalTestCase):

    def test_get_compute_usage(self):
        '''Test usage functionality'''
        start = datetime.datetime.now() - datetime.timedelta(seconds=5)
        usage = self.operator_cloud.get_compute_usage('demo', start)
        self.add_info_on_exception('usage', usage)
        self.assertIsNotNone(usage)
        self.assertIn('total_hours', usage)
        self.assertIn('started_at', usage)
        self.assertEqual(start.isoformat(), usage['started_at'])
        self.assertIn('location', usage)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_floating_ip_pool
----------------------------------

Test floating IP pool resource (managed by nova)
"""

from shade import OpenStackCloudException
from shade.tests.unit import base
from shade.tests import fakes


class TestFloatingIPPool(base.RequestsMockTestCase):
    pools = [{'name': u'public'}]

    def test_list_floating_ip_pools(self):

        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/extensions'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'extensions': [{
                     u'alias': u'os-floating-ip-pools',
                     u'updated': u'2014-12-03T00:00:00Z',
                     u'name': u'FloatingIpPools',
                     u'links': [],
                     u'namespace':
                     u'http://docs.openstack.org/compute/ext/fake_xml',
                     u'description': u'Floating IPs support.'}]}),
            dict(method='GET',
                 uri='{endpoint}/os-floating-ip-pools'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={"floating_ip_pools": [{"name": "public"}]})
        ])

        floating_ip_pools = self.cloud.list_floating_ip_pools()

        self.assertItemsEqual(floating_ip_pools, self.pools)

        self.assert_calls()

    def test_list_floating_ip_pools_exception(self):

        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/extensions'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={'extensions': [{
                     u'alias': u'os-floating-ip-pools',
                     u'updated': u'2014-12-03T00:00:00Z',
                     u'name': u'FloatingIpPools',
                     u'links': [],
                     u'namespace':
                     u'http://docs.openstack.org/compute/ext/fake_xml',
                     u'description': u'Floating IPs support.'}]}),
            dict(method='GET',
                 uri='{endpoint}/os-floating-ip-pools'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 status_code=404)])

        self.assertRaises(
            OpenStackCloudException, self.cloud.list_floating_ip_pools)

        self.assert_calls()



# Copyright (c) 2017 Red Hat, Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from shade.tests.unit import base
from shade.tests import fakes


_fake_zone_list = {
    "availabilityZoneInfo": [
        {
            "hosts": None,
            "zoneName": "az1",
            "zoneState": {
                "available": True
            }
        },
        {
            "hosts": None,
            "zoneName": "nova",
            "zoneState": {
                "available": False
            }
        }
    ]
}


class TestAvailabilityZoneNames(base.RequestsMockTestCase):

    def test_list_availability_zone_names(self):
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/os-availability-zone'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json=_fake_zone_list),
        ])

        self.assertEqual(
            ['az1'], self.cloud.list_availability_zone_names())

        self.assert_calls()

    def test_unauthorized_availability_zone_names(self):
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/os-availability-zone'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 status_code=403),
        ])

        self.assertEqual(
            [], self.cloud.list_availability_zone_names())

        self.assert_calls()

    def test_list_all_availability_zone_names(self):
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/os-availability-zone'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json=_fake_zone_list),
        ])

        self.assertEqual(
            ['az1', 'nova'],
            self.cloud.list_availability_zone_names(unavailable=True))

        self.assert_calls()



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_floating_ip_common
----------------------------------

Tests floating IP resource methods for Neutron and Nova-network.
"""

from mock import patch

from shade import exc
from shade import meta
from shade import OpenStackCloud
from shade.tests.fakes import FakeServer
from shade.tests.unit import base


class TestFloatingIP(base.TestCase):

    @patch.object(OpenStackCloud, 'get_floating_ip')
    @patch.object(OpenStackCloud, '_attach_ip_to_server')
    @patch.object(OpenStackCloud, 'available_floating_ip')
    def test_add_auto_ip(
            self, mock_available_floating_ip, mock_attach_ip_to_server,
            mock_get_floating_ip):
        server = FakeServer(
            id='server-id', name='test-server', status="ACTIVE", addresses={}
        )
        server_dict = meta.obj_to_dict(server)
        floating_ip_dict = {
            "id": "this-is-a-floating-ip-id",
            "fixed_ip_address": None,
            "internal_network": None,
            "floating_ip_address": "203.0.113.29",
            "network": "this-is-a-net-or-pool-id",
            "attached": False,
            "status": "ACTIVE"
        }

        mock_available_floating_ip.return_value = floating_ip_dict

        self.cloud.add_auto_ip(server=server_dict)

        mock_attach_ip_to_server.assert_called_with(
            timeout=60, wait=False, server=server_dict,
            floating_ip=floating_ip_dict, skip_attach=False)

    @patch.object(OpenStackCloud, 'nova_client')
    @patch.object(OpenStackCloud, '_add_ip_from_pool')
    def test_add_ips_to_server_pool(
            self, mock_add_ip_from_pool, mock_nova_client):
        server = FakeServer(
            id='romeo', name='test-server', status="ACTIVE", addresses={}
        )
        server_dict = meta.obj_to_dict(server)
        pool = 'nova'

        mock_nova_client.servers.get.return_value = server

        self.cloud.add_ips_to_server(server_dict, ip_pool=pool)

        mock_add_ip_from_pool.assert_called_with(
            server_dict, pool, reuse=True, wait=False, timeout=60,
            fixed_address=None, nat_destination=None)

    @patch.object(OpenStackCloud, 'has_service')
    @patch.object(OpenStackCloud, 'get_floating_ip')
    @patch.object(OpenStackCloud, '_add_auto_ip')
    def test_add_ips_to_server_ipv6_only(
            self, mock_add_auto_ip,
            mock_get_floating_ip,
            mock_has_service):
        self.cloud._floating_ip_source = None
        self.cloud.force_ipv4 = False
        self.cloud._local_ipv6 = True
        mock_has_service.return_value = False
        server = FakeServer(
            id='server-id', name='test-server', status="ACTIVE",
            addresses={
                'private': [{
                    'addr': "10.223.160.141",
                    'version': 4
                }],
                'public': [{
                    u'OS-EXT-IPS-MAC:mac_addr': u'fa:16:3e:ae:7d:42',
                    u'OS-EXT-IPS:type': u'fixed',
                    'addr': "2001:4800:7819:103:be76:4eff:fe05:8525",
                    'version': 6
                }]
            }
        )
        server_dict = meta.add_server_interfaces(
            self.cloud, meta.obj_to_dict(server))

        new_server = self.cloud.add_ips_to_server(server=server_dict)
        mock_get_floating_ip.assert_not_called()
        mock_add_auto_ip.assert_not_called()
        self.assertEqual(
            new_server['interface_ip'],
            '2001:4800:7819:103:be76:4eff:fe05:8525')
        self.assertEqual(new_server['private_v4'], '10.223.160.141')
        self.assertEqual(new_server['public_v4'], '')
        self.assertEqual(
            new_server['public_v6'], '2001:4800:7819:103:be76:4eff:fe05:8525')

    @patch.object(OpenStackCloud, 'has_service')
    @patch.object(OpenStackCloud, 'get_floating_ip')
    @patch.object(OpenStackCloud, '_add_auto_ip')
    def test_add_ips_to_server_rackspace(
            self, mock_add_auto_ip,
            mock_get_floating_ip,
            mock_has_service):
        self.cloud._floating_ip_source = None
        self.cloud.force_ipv4 = False
        self.cloud._local_ipv6 = True
        mock_has_service.return_value = False
        server = FakeServer(
            id='server-id', name='test-server', status="ACTIVE",
            addresses={
                'private': [{
                    'addr': "10.223.160.141",
                    'version': 4
                }],
                'public': [{
                    'addr': "104.130.246.91",
                    'version': 4
                }, {
                    'addr': "2001:4800:7819:103:be76:4eff:fe05:8525",
                    'version': 6
                }]
            }
        )
        server_dict = meta.add_server_interfaces(
            self.cloud, meta.obj_to_dict(server))

        new_server = self.cloud.add_ips_to_server(server=server_dict)
        mock_get_floating_ip.assert_not_called()
        mock_add_auto_ip.assert_not_called()
        self.assertEqual(
            new_server['interface_ip'],
            '2001:4800:7819:103:be76:4eff:fe05:8525')

    @patch.object(OpenStackCloud, 'has_service')
    @patch.object(OpenStackCloud, 'get_floating_ip')
    @patch.object(OpenStackCloud, '_add_auto_ip')
    def test_add_ips_to_server_rackspace_local_ipv4(
            self, mock_add_auto_ip,
            mock_get_floating_ip,
            mock_has_service):
        self.cloud._floating_ip_source = None
        self.cloud.force_ipv4 = False
        self.cloud._local_ipv6 = False
        mock_has_service.return_value = False
        server = FakeServer(
            id='server-id', name='test-server', status="ACTIVE",
            addresses={
                'private': [{
                    'addr': "10.223.160.141",
                    'version': 4
                }],
                'public': [{
                    'addr': "104.130.246.91",
                    'version': 4
                }, {
                    'addr': "2001:4800:7819:103:be76:4eff:fe05:8525",
                    'version': 6
                }]
            }
        )
        server_dict = meta.add_server_interfaces(
            self.cloud, meta.obj_to_dict(server))

        new_server = self.cloud.add_ips_to_server(server=server_dict)
        mock_get_floating_ip.assert_not_called()
        mock_add_auto_ip.assert_not_called()
        self.assertEqual(new_server['interface_ip'], '104.130.246.91')

    @patch.object(OpenStackCloud, 'nova_client')
    @patch.object(OpenStackCloud, 'add_ip_list')
    def test_add_ips_to_server_ip_list(
            self, mock_add_ip_list, mock_nova_client):
        server = FakeServer(
            id='server-id', name='test-server', status="ACTIVE", addresses={}
        )
        server_dict = meta.obj_to_dict(server)
        ips = ['203.0.113.29', '172.24.4.229']
        mock_nova_client.servers.get.return_value = server

        self.cloud.add_ips_to_server(server_dict, ips=ips)

        mock_add_ip_list.assert_called_with(
            server_dict, ips, wait=False, timeout=60, fixed_address=None)

    @patch.object(OpenStackCloud, '_needs_floating_ip')
    @patch.object(OpenStackCloud, 'nova_client')
    @patch.object(OpenStackCloud, '_add_auto_ip')
    def test_add_ips_to_server_auto_ip(
            self, mock_add_auto_ip, mock_nova_client, mock_needs_floating_ip):
        server = FakeServer(
            id='server-id', name='test-server', status="ACTIVE", addresses={}
        )
        server_dict = meta.obj_to_dict(server)

        mock_nova_client.servers.get.return_value = server
        # TODO(mordred) REMOVE THIS MOCK WHEN THE NEXT PATCH LANDS
        # SERIOUSLY THIS TIME. NEXT PATCH - WHICH SHOULD ADD MOCKS FOR
        # list_ports AND list_networks AND list_subnets. BUT THAT WOULD
        # BE NOT ACTUALLY RELATED TO THIS PATCH. SO DO IT NEXT PATCH
        mock_needs_floating_ip.return_value = True

        self.cloud.add_ips_to_server(server_dict)

        mock_add_auto_ip.assert_called_with(
            server_dict, wait=False, timeout=60, reuse=True)

    @patch.object(OpenStackCloud, 'search_ports', return_value=[{}])
    def test_nat_destination_port_when_no_free_fixed_ip(
            self, mock_search_ports):
        server = {'id': 42}
        self.assertRaisesRegexp(
            exc.OpenStackCloudException, 'server 42$',
            self.cloud._nat_destination_port, server
        )



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_cloud_services
----------------------------------

Tests Keystone services commands.
"""

from shade import OpenStackCloudException
from shade.exc import OpenStackCloudUnavailableFeature
from shade.tests.unit import base
from testtools import matchers


class CloudServices(base.RequestsMockTestCase):

    def setUp(self, cloud_config_fixture='clouds.yaml'):
        super(CloudServices, self).setUp(cloud_config_fixture)

    def get_mock_url(self, service_type='identity', interface='admin',
                     resource='services', append=None, base_url_append='v3'):

        return super(CloudServices, self).get_mock_url(
            service_type, interface, resource, append, base_url_append)

    def test_create_service_v2(self):
        self.use_keystone_v2()
        service_data = self._get_service_data(name='a service', type='network',
                                              description='A test service')
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(base_url_append='OS-KSADM'),
                 status_code=200,
                 json=service_data.json_response_v2,
                 validate=service_data.json_request),
            dict(method='GET',
                 uri=self.get_mock_url(base_url_append='OS-KSADM',
                                       append=[service_data.service_id]),
                 status_code=200,
                 json=service_data.json_response_v2)
        ])

        service = self.op_cloud.create_service(
            name=service_data.service_name,
            service_type=service_data.service_type,
            description=service_data.description)
        self.assertThat(service.name,
                        matchers.Equals(service_data.service_name))
        self.assertThat(service.id, matchers.Equals(service_data.service_id))
        self.assertThat(service.description,
                        matchers.Equals(service_data.description))
        self.assertThat(service.type,
                        matchers.Equals(service_data.service_type))
        self.assert_calls()

    def test_create_service_v3(self):
        self._add_discovery_uri_call()
        service_data = self._get_service_data(name='a service', type='network',
                                              description='A test service')
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json=service_data.json_response_v3,
                 validate=service_data.json_request),
            dict(method='GET',
                 uri=self.get_mock_url(append=[service_data.service_id]),
                 status_code=200,
                 json=service_data.json_response_v3)
        ])

        service = self.op_cloud.create_service(
            name=service_data.service_name,
            service_type=service_data.service_type,
            description=service_data.description)
        self.assertThat(service.name,
                        matchers.Equals(service_data.service_name))
        self.assertThat(service.id, matchers.Equals(service_data.service_id))
        self.assertThat(service.description,
                        matchers.Equals(service_data.description))
        self.assertThat(service.type,
                        matchers.Equals(service_data.service_type))
        self.assert_calls()

    def test_update_service_v2(self):
        self.use_keystone_v2()
        # NOTE(SamYaple): Update service only works with v3 api
        self.assertRaises(OpenStackCloudUnavailableFeature,
                          self.op_cloud.update_service,
                          'service_id', name='new name')

    def test_update_service_v3(self):
        self._add_discovery_uri_call()
        service_data = self._get_service_data(name='a service', type='network',
                                              description='A test service')
        request = service_data.json_request.copy()
        request['enabled'] = False
        resp = service_data.json_response_v3.copy()
        resp['enabled'] = False
        self.register_uris([
            dict(method='PATCH',
                 uri=self.get_mock_url(append=[service_data.service_id]),
                 status_code=200,
                 json=resp,
                 validate=request),
            dict(method='GET',
                 uri=self.get_mock_url(append=[service_data.service_id]),
                 status_code=200,
                 json=resp),
        ])

        service = self.op_cloud.update_service(service_data.service_id,
                                               enabled=False)
        self.assertThat(service.name,
                        matchers.Equals(service_data.service_name))
        self.assertThat(service.id, matchers.Equals(service_data.service_id))
        self.assertThat(service.description,
                        matchers.Equals(service_data.description))
        self.assertThat(service.type,
                        matchers.Equals(service_data.service_type))
        self.assert_calls()

    def test_list_services(self):
        self._add_discovery_uri_call()
        service_data = self._get_service_data()
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [service_data.json_response_v3['service']]})
        ])
        services = self.op_cloud.list_services()
        self.assertThat(len(services), matchers.Equals(1))
        self.assertThat(services[0].id,
                        matchers.Equals(service_data.service_id))
        self.assertThat(services[0].name,
                        matchers.Equals(service_data.service_name))
        self.assertThat(services[0].type,
                        matchers.Equals(service_data.service_type))
        self.assert_calls()

    def test_get_service(self):
        self._add_discovery_uri_call()
        service_data = self._get_service_data()
        service2_data = self._get_service_data()
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [
                     service_data.json_response_v3['service'],
                     service2_data.json_response_v3['service']]}),
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [
                     service_data.json_response_v3['service'],
                     service2_data.json_response_v3['service']]}),
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [
                     service_data.json_response_v3['service'],
                     service2_data.json_response_v3['service']]}),
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=400),
        ])

        # Search by id
        service = self.op_cloud.get_service(name_or_id=service_data.service_id)
        self.assertThat(service.id, matchers.Equals(service_data.service_id))

        # Search by name
        service = self.op_cloud.get_service(
            name_or_id=service_data.service_name)
        # test we are getting exactly 1 element
        self.assertThat(service.id, matchers.Equals(service_data.service_id))

        # Not found
        service = self.op_cloud.get_service(name_or_id='INVALID SERVICE')
        self.assertIs(None, service)

        # Multiple matches
        # test we are getting an Exception
        self.assertRaises(OpenStackCloudException, self.op_cloud.get_service,
                          name_or_id=None, filters={'type': 'type2'})
        self.assert_calls()

    def test_search_services(self):
        self._add_discovery_uri_call()
        service_data = self._get_service_data()
        service2_data = self._get_service_data(type=service_data.service_type)
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [
                     service_data.json_response_v3['service'],
                     service2_data.json_response_v3['service']]}),
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [
                     service_data.json_response_v3['service'],
                     service2_data.json_response_v3['service']]}),
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [
                     service_data.json_response_v3['service'],
                     service2_data.json_response_v3['service']]}),
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [
                     service_data.json_response_v3['service'],
                     service2_data.json_response_v3['service']]}),
        ])

        # Search by id
        services = self.op_cloud.search_services(
            name_or_id=service_data.service_id)
        # test we are getting exactly 1 element
        self.assertThat(len(services), matchers.Equals(1))
        self.assertThat(services[0].id,
                        matchers.Equals(service_data.service_id))

        # Search by name
        services = self.op_cloud.search_services(
            name_or_id=service_data.service_name)
        # test we are getting exactly 1 element
        self.assertThat(len(services), matchers.Equals(1))
        self.assertThat(services[0].name,
                        matchers.Equals(service_data.service_name))

        # Not found
        services = self.op_cloud.search_services(name_or_id='!INVALID!')
        self.assertThat(len(services), matchers.Equals(0))

        # Multiple matches
        services = self.op_cloud.search_services(
            filters={'type': service_data.service_type})
        # test we are getting exactly 2 elements
        self.assertThat(len(services), matchers.Equals(2))
        self.assertThat(services[0].id,
                        matchers.Equals(service_data.service_id))
        self.assertThat(services[1].id,
                        matchers.Equals(service2_data.service_id))
        self.assert_calls()

    def test_delete_service(self):
        self._add_discovery_uri_call()
        service_data = self._get_service_data()
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [
                     service_data.json_response_v3['service']]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(append=[service_data.service_id]),
                 status_code=204),
            dict(method='GET',
                 uri=self.get_mock_url(),
                 status_code=200,
                 json={'services': [
                     service_data.json_response_v3['service']]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(append=[service_data.service_id]),
                 status_code=204)
        ])

        # Delete by name
        self.op_cloud.delete_service(name_or_id=service_data.service_name)

        # Delete by id
        self.op_cloud.delete_service(service_data.service_id)

        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import mock

import shade
from shade.tests.unit import base
from shade.tests import fakes


class TestServerGroup(base.TestCase):

    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
    def test_create_server_group(self, mock_nova):
        server_group_name = 'my-server-group'
        self.cloud.create_server_group(name=server_group_name,
                                       policies=['affinity'])

        mock_nova.server_groups.create.assert_called_once_with(
            name=server_group_name, policies=['affinity']
        )

    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
    def test_delete_server_group(self, mock_nova):
        mock_nova.server_groups.list.return_value = [
            fakes.FakeServerGroup('1234', 'name', ['affinity'])
        ]
        self.assertTrue(self.cloud.delete_server_group('1234'))
        mock_nova.server_groups.list.assert_called_once_with()
        mock_nova.server_groups.delete.assert_called_once_with(
            id='1234'
        )
